#include <PulseOximeter.h>
#include <WebServer.h>

#define BUTTON_PIN 2
#define PULSEOXIMETER_PIN A0

PulseOximeter pulseOximeter;
WebServer server(80);

enum State {
  INIT,
  MEASURE,
};

State currentState = INIT;
bool buttonPressed = false;
int lastAchievedHeartRate = 0; // Variable to store the last achieved heart rate

const int NUM_READINGS = 10;
int heartRateReadings[NUM_READINGS];
int currentIndex = 0;
unsigned long lastUpdateMillis = 0;
const int UPDATE_INTERVAL = 5000;

void handleRoot() {
  String buttonLabel = (currentState == INIT) ? "Start Measurement" : "Stop Measurement";
  String html = "<html><body>";
  html += "<h1>ESP32 Pulse Oximeter</h1>";
  html += "<p>Current State: ";
  html += (currentState == INIT) ? "INIT" : "MEASURE";
  html += "</p>";
  html += "<form action=\"/start\" method=\"post\">";
  html += "<button type=\"submit\">" + buttonLabel + "</button>";
  html += "</form>";
  html += "<p>Last Achieved Heart Rate: " + String(lastAchievedHeartRate) + "</p>";
  html += "</body></html>";
  server.send(200, "text/html", html);
}

void handleStart() {
  if (currentState == INIT) {
    currentState = MEASURE;
    buttonPressed = true;
    pulseOximeter.begin();
    server.sendHeader("Location", "/");
  } else {
    currentState = INIT;
    buttonPressed = false;
    pulseOximeter.end();
    lastAchievedHeartRate = 0; // Reset last achieved heart rate when stopping measurement
    server.sendHeader("Location", "/");
  }
  server.send(302, "text/plain", "");
}

void setup() {
  Serial.begin(115200);

  pinMode(BUTTON_PIN, INPUT_PULLUP);

  for (int i = 0; i < NUM_READINGS; i++) {
    heartRateReadings[i] = 0;
  }

  // Set up web server routes
  server.on("/", HTTP_GET, handleRoot);
  server.on("/start", HTTP_POST, handleStart);

  // Start the web server
  server.begin();
}

void loop() {
  server.handleClient();

  // Your existing code for pulse oximeter measurements and state transitions
  // ...

  // At the end of your MEASURE code
  if (currentState == MEASURE && validHeartRateAchieved()) {
    // Perform any actions for a valid heart rate
    lastAchievedHeartRate = calculateAverageHeartRate(); // Update the last achieved heart rate
    currentState = INIT; // Switch back to INIT state
  }
}

bool validHeartRateAchieved() {
  // Implement your criteria for a valid heart rate
  // For example, check if the running average is within a certain range
  // You can customize this based on your specific requirements
  return (calculateAverageHeartRate() > 0 && calculateAverageHeartRate() < 200);
}

int calculateAverageHeartRate() {
  // Calculate the average heart rate using your existing logic
  // ...
}
